package com.steerinfo.dil.service.impl;

import com.steerinfo.dil.mapper.BmsshipFeeMapper;
import com.steerinfo.dil.mapper.BmsshipStatementMapper;
import com.steerinfo.dil.model.BmsshipDetailsOrder;
import com.steerinfo.dil.model.BmsshipDetailsStatement;
import com.steerinfo.dil.model.BmsshipFee;
import com.steerinfo.dil.model.BmsshipStatement;
import com.steerinfo.dil.service.IBmsshipStatementService;
import com.steerinfo.dil.util.DataChange;
import com.steerinfo.dil.util.NoUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.*;

/**
 * @Description:
 * @Author:HuJianGuo
 * @GreateTime:2021/9/8 19:26
 * @Version:V2.0
 */
@Service
public class BmsshipStatementServiceImpl implements IBmsshipStatementService {

    @Autowired
    BmsshipStatementMapper bmsshipStatementMapper;
    @Autowired
    BmsshipDetailsStatementServiceImpl bmsshipDetailsStatementService;
    @Autowired
    BmsshipDetailsOrderSerivceImpl bmsshipDetailsOrderSerivce;
    @Autowired
    BmstruckFormulaServiceImpl bmstruckFormulaService;
    @Autowired
    NoUtil noUtil;
    @Autowired
    BmsshipFeeMapper bmsshipFeeMapper;

    /**
     * 生成水运费账单
     * @param listMap
     * @return
     */
    @Override
    public int insertSelective(List<Map<String, Object>> listMap) throws Exception {
        // 生成水运费账单
        BmsshipStatement bmsshipStatement = new BmsshipStatement();
        BigDecimal statementId = selectMaxId();
        bmsshipStatement.setStatementId(statementId);
        String statementNo = noUtil.setResultNo("CYZD",statementId);
        bmsshipStatement.setStatementNo(statementNo);
        BigDecimal statementShipperId = new BigDecimal((Integer) listMap.get(0).get("statementShipperId"));
        bmsshipStatement.setStatementShipperId(statementShipperId);
        bmsshipStatement.setStatementDate(new Date());
        // 设置常规字段
        bmsshipStatement.setInsertTime(new Date());
        bmsshipStatement.setInsertUsername("admin");
        bmsshipStatement.setUpdateTime(new Date());
        bmsshipStatement.setUpdateUsername("admin");
        bmsshipStatement.setInsertUpdateRemark("无");
        Double doubleTotalAmount = new Double(0);
        // 详单-账单
        List<BmsshipDetailsStatement> list = new LinkedList<>();
        for (int i = 0;i < listMap.size(); i++) {
            BigDecimal detailsStatementId = bmsshipDetailsStatementService.selectMaxId();
            BigDecimal detailsId = new BigDecimal((Integer) listMap.get(i).get("detailsId"));
            BmsshipDetailsOrder bmsshipDetailsOrder = bmsshipDetailsOrderSerivce.selectByPrimaryKey(detailsId);
            bmsshipDetailsOrder.setWetherToStatement(new BigDecimal(1));
            bmsshipDetailsOrderSerivce.updateByPrimaryKeySelective(bmsshipDetailsOrder);
            BmsshipDetailsStatement bmsshipDetailsStatement = new BmsshipDetailsStatement();
            bmsshipDetailsStatement.setDetailsStatementId(detailsStatementId);
            bmsshipDetailsStatement.setStatementId(statementId);
            bmsshipDetailsStatement.setDetailsId(detailsId);
            // 设置常规字段
            bmsshipDetailsStatement.setInsertTime(new Date());
            bmsshipDetailsStatement.setInsertUsername("admin");
            bmsshipDetailsStatement.setUpdateTime(new Date());
            bmsshipDetailsStatement.setUpdateUsername("admin");
            bmsshipDetailsStatement.setInsertUpdateRemark("无");
            list.add(bmsshipDetailsStatement);
            Object detailsAmount = listMap.get(i).get("detailsAmount");
            Double detailsAmountDouble = Double.valueOf(detailsAmount.toString());
            doubleTotalAmount += detailsAmountDouble;
            bmsshipDetailsStatementService.insertSelective(bmsshipDetailsStatement);
        }
        DecimalFormat df = new DecimalFormat("0.00");
        String statementTotalAmount = df.format(doubleTotalAmount);
        bmsshipStatement.setStatementTotalAmount(new BigDecimal(statementTotalAmount));
        bmsshipStatement.setStatementType("水运费");
        int code = bmsshipStatementMapper.insertSelective(bmsshipStatement);
        return code;
    }

    /**
     * 生成质检费账单
     * @param mapList
     * @return
     */
    public int insertWaterFeesStatement(List<Map<String,Object>> mapList) throws Exception {
        // 生成水运费账单

        BmsshipStatement bmsshipStatement = new BmsshipStatement();
        BigDecimal statementId = selectMaxId();
        bmsshipStatement.setStatementId(statementId);
        String statementNo = noUtil.setResultNo("ZD",statementId);
        bmsshipStatement.setStatementNo(statementNo);
        bmsshipStatement.setStatementDate(new Date());
        BigDecimal doubleTotalAmount = bmstruckFormulaService.generateWaterFees(mapList);
        // 详单-账单
        DecimalFormat df = new DecimalFormat("0.00");
        String statementTotalAmount = df.format(doubleTotalAmount.doubleValue());
        bmsshipStatement.setStatementTotalAmount(new BigDecimal(statementTotalAmount));
        bmsshipStatement.setStatementType("水分检测费");
        // 设置常规字段
        bmsshipStatement.setInsertTime(new Date());
        bmsshipStatement.setInsertUsername("admin");
        bmsshipStatement.setUpdateTime(new Date());
        bmsshipStatement.setUpdateUsername("admin");
        bmsshipStatement.setInsertUpdateRemark("无");
        bmsshipStatement.setStatementShipperId(new BigDecimal(1));
        int code = bmsshipStatementMapper.insertSelective(bmsshipStatement);
        return code;
    }

    /**
     * 生成物流途损超标扣款账单
     * @param batchId
     * @return
     */
    public int insertLossStatement(BigDecimal batchId) throws Exception {
        // 生成水运费账单
        BmsshipStatement bmsshipStatement = new BmsshipStatement();
        BigDecimal statementId = selectMaxId();
        bmsshipStatement.setStatementId(statementId);
        String statementNo = noUtil.setResultNo("ZD",statementId);
        bmsshipStatement.setStatementNo(statementNo);
        bmsshipStatement.setStatementDate(new Date());
        bmsshipStatement.setStatementShipperId(new BigDecimal(1));
        BigDecimal doubleTotalAmount = bmstruckFormulaService.generateLossCost(batchId);
        // 详单-账单
        DecimalFormat df = new DecimalFormat("0.00");
        String statementTotalAmount = df.format(doubleTotalAmount.doubleValue());
        bmsshipStatement.setStatementTotalAmount(new BigDecimal(statementTotalAmount));
        bmsshipStatement.setStatementType("物流途损超标扣款");
        // 设置常规字段
        bmsshipStatement.setInsertTime(new Date());
        bmsshipStatement.setInsertUsername("admin");
        bmsshipStatement.setUpdateTime(new Date());
        bmsshipStatement.setUpdateUsername("admin");
        bmsshipStatement.setInsertUpdateRemark("无");
        int code = bmsshipStatementMapper.insertSelective(bmsshipStatement);
        return code;
    }

    /**
     * 得到最大id
     * @return
     */
    @Override
    public BigDecimal selectMaxId() {
        BigDecimal id = bmsshipStatementMapper.selectMaxId();
        return id == null ? new BigDecimal(1): new BigDecimal(id.intValue() + 1);
    }

    /**
     * 展示账单信息
     * @param mapValue
     * @return
     */
    @Override
    public List<Map<String, Object>> getStatementList(Map<String, Object> mapValue) {
        return bmsshipStatementMapper.getStatementList(mapValue);
    }

    /**
     * 展示账单中的详单
     * @param mapValue
     * @return
     */
    @Override
    public List<Map<String, Object>> getDetailsOrder(Map<String, Object> mapValue) {
        return bmsshipStatementMapper.getDetailsOrder(mapValue);
    }

    /**
     * 得到总金额和支付单位
     * @param batchId
     * @return
     */
    @Override
    public List<Map<String, Object>> getShipperAndAmount(BigDecimal batchId) {
        return bmsshipStatementMapper.getShipperAndAmount(batchId);
    }

    /**
     * 得到汽运实绩
     * @param batchId
     * @return
     */
    @Override
    public List<Map<String, Object>> getTruckResult(BigDecimal batchId) {
        return bmsshipStatementMapper.getTruckResult(batchId);
    }

    /**
     * 得到船运实绩
     * @param mapVal
     * @return
     */
    @Override
    public List<Map<String, Object>> getShipResult(Map<String,Object> mapVal) {
        return bmsshipStatementMapper.getShipResult(mapVal);
    }

    /**
     * 展示途损费账单列表
     * @param mapValue
     * @return
     */
    @Override
    public List<Map<String, Object>> getLossStatementList(Map<String, Object> mapValue) {
        return bmsshipStatementMapper.getLossStatementList(mapValue);
    }

    /**
     * 展示水分检测实绩
     * @param mapValue
     * @return
     */
    @Override
    public List<Map<String, Object>> getWaterResult(Map<String, Object> mapValue) {
        return bmsshipStatementMapper.getWaterResult(mapValue);
    }

    /**
     * 得到批次信息
     * @param mapVal
     * @return
     */
    @Override
    public List<Map<String, Object>> getBatch(Map<String, Object> mapVal) {
        return bmsshipStatementMapper.getBatch(mapVal);
    }

    @Override
    public int addwaterStatement(Map<String, Object> map) {
        BigDecimal fee=null;
        String userId =(String) map.get("userId");
        BigDecimal portNum = DataChange.dataToBigDecimal(map.get("portNum"));
        BigDecimal shipNum =DataChange.dataToBigDecimal(map.get("shipNum"));
        BigDecimal unitPrice =DataChange.dataToBigDecimal(map.get("unitPrice"));
        BigDecimal unitPriceId =DataChange.dataToBigDecimal(map.get("unitPriceId"));
        BigDecimal feeType =DataChange.dataToBigDecimal(map.get("feeType"));
        BigDecimal batchId =DataChange.dataToBigDecimal(map.get("batchId"));
        BigDecimal purchaseOrderId =DataChange.dataToBigDecimal(map.get("purchaseOrderId"));
        BmsshipFee bmsshipFee =new BmsshipFee();
        if (shipNum!=null&&shipNum.intValue()!=0){
            //无拼装
            fee=shipNum.multiply(unitPrice);
            bmsshipFee.setShipNum(shipNum);
        }else if (portNum!=null&&portNum.intValue()!=0){
            fee=portNum.multiply(unitPrice);
            bmsshipFee.setPortNum(portNum);
        }
        bmsshipFee.setResultId(bmsshipFeeMapper.MaxId());
        bmsshipFee.setFeeType(feeType);
        bmsshipFee.setUnitPriceId(unitPriceId);
        bmsshipFee.setFee(fee);
        bmsshipFee.setDeleted(new BigDecimal(0));
        bmsshipFee.setPurchaseOrderId(purchaseOrderId);
        bmsshipFee.setUnitPrice(unitPrice);
        bmsshipFee.setInsertUsername(userId);
        bmsshipFee.setInsertTime(new Date());
        bmsshipFee.setBatchId(batchId);
        long makeTime =(long) map.get("makeTime");
        bmsshipFee.setMakeTime(new Date(makeTime));
        bmsshipFee.setInsertUpdateRemark("水分检测费");
        return bmsshipFeeMapper.insertSelective(bmsshipFee);
    }
}
